@@ -7,8 +7,8 @@ from jax import numpy as jnp
 import pgx.core as core
 from pgx.flax.struct import dataclass
 
-BLACK = 0
-WHITE = 1
+BLACK = 1
+WHITE = -1
 POINT = 2
 BLACK_CHAR = "@"
 WHITE_CHAR = "O"
@@ -28,18 +28,16 @@ class State(core.State):
     terminated: jnp.ndarray = FALSE
     legal_action_mask: jnp.ndarray = jnp.zeros(19 * 19 + 1, dtype=jnp.bool_)
     observation: jnp.ndarray = jnp.zeros((17, 19, 19), dtype=jnp.bool_)
-
-    # 横幅, マスの数ではない
-    size: jnp.ndarray = jnp.int32(19)  # type:ignore
+    # ---
+    size: jnp.ndarray = jnp.int32(19)  # NOTE: require 19 * 19 > int8
 
     # 連の代表点（一番小さいマス目）のマス目の座標
-    ren_id_board: jnp.ndarray = jnp.full(
-        (2, 19 * 19), -1, dtype=jnp.int32
-    )  # type:ignore
+    # NOTE: require at least 19 * 19 > int8, idx_squared_sum can be 361^2 > int16
+    ren_id_board: jnp.ndarray = jnp.zeros(19 * 19, dtype=jnp.int32)
 
     # 直近8回のログ
     game_log: jnp.ndarray = jnp.full(
-        (8, 19 * 19), 2, dtype=jnp.int32
+        (8, 19 * 19), 2, dtype=jnp.int8
     )  # type:ignore
 
     # 経過ターン, 0始まり
@@ -55,16 +53,17 @@ class State(core.State):
     kou: jnp.ndarray = jnp.int32(-1)  # type:ignore
 
     # コミ
-    komi: jnp.ndarray = jnp.float32(6.5)  # type:ignore
+    komi: jnp.ndarray = jnp.float32(7.5)  # type:ignore
 
 
 class Go(core.Env):
-    def __init__(self, size: int = 19):
+    def __init__(self, size: int = 19, komi: float = 7.5):
         super().__init__()
         self.size = size
+        self.komi = komi
 
     def _init(self, key: jax.random.KeyArray) -> State:
-        return partial(init, size=self.size)(key=key)
+        return partial(init, size=self.size, komi=self.komi)(key=key)
 
     def _step(self, state: core.State, action: jnp.ndarray) -> State:
         assert isinstance(state, State)
@@ -85,11 +84,11 @@ class Go(core.Env):
         return -1.0, 1.0
 
 
-def observe(state: State, player_id, observe_all=False):
-    return _get_alphazero_features(state, player_id, observe_all)
+def observe(state: State, player_id):
+    return _get_alphazero_features(state, player_id)
 
 
-def _get_alphazero_features(state: State, player_id, observe_all):
+def _get_alphazero_features(state: State, player_id):
     """
     17 x (size x size)
     0: player_idの石
@@ -143,15 +142,14 @@ def _get_alphazero_features(state: State, player_id, observe_all):
     return jnp.vstack([log, color])
 
 
-def init(key: jax.random.KeyArray, size: int) -> State:
+def init(key: jax.random.KeyArray, size: int, komi: float = 7.5) -> State:
     return State(  # type:ignore
-        size=jnp.int32(size),
-        ren_id_board=jnp.full(
-            (2, size**2), -1, dtype=jnp.int32
-        ),  # type:ignore
+        size=jnp.int32(size),  # type:ignore
+        ren_id_board=jnp.zeros(size**2, dtype=jnp.int32),
         legal_action_mask=jnp.ones(size**2 + 1, dtype=jnp.bool_),
-        game_log=jnp.full((8, size**2), 2, dtype=jnp.int32),
+        game_log=jnp.full((8, size**2), 2, dtype=jnp.int8),
         curr_player=jnp.int8(jax.random.bernoulli(key)),
+        komi=jnp.float32(komi),
     )
 
 
@@ -170,9 +168,7 @@ def step(state: State, action: int, size: int) -> State:
     # update log
     new_log = jnp.roll(_state.game_log, size**2)
     new_log = new_log.at[0].set(get_board(_state))
-    _state = _state.replace(game_log=new_log)  # type:ignore
-
-    return _state
+    return _state.replace(game_log=new_log)  # type:ignore
 
 
 def _update_state_wo_legal_action(
@@ -197,41 +193,40 @@ def _pass_move(_state: State, _size: int) -> State:
     return jax.lax.cond(
         _state.passed,
         # 連続でパスならば終局
-        lambda: _state.replace(terminated=TRUE, reward=_get_reward(_state, _size)),  # type: ignore
+        lambda: _state.replace(terminated=TRUE, kou=jnp.int32(-1), reward=_get_reward(_state, _size)),  # type: ignore
         # 1回目のパスならばStateにパスを追加してそのまま続行
-        lambda: _state.replace(passed=True, reward=jnp.zeros(2, dtype=jnp.float32)),  # type: ignore
+        lambda: _state.replace(passed=True, kou=jnp.int32(-1), reward=jnp.zeros(2, dtype=jnp.float32)),  # type: ignore
     )
 
 
 def _not_pass_move(_state: State, _action: int, size) -> State:
     state = _state.replace(passed=FALSE)  # type: ignore
     xy = _action
-    my_color = _my_color(state)
-    agehama_before = state.agehama[my_color]
+    my_color_ix = _my_color_ix(state)
+    agehama_before = state.agehama[my_color_ix]
 
-    kou_occurred = _kou_occurred(state, xy)
+    kou_may_occur = _kou_may_occur(state, xy)
 
     # 周囲の連から敵石を除く
     adj_xy = _neighbour(xy, size)
     oppo_color = _opponent_color(state)
-    oppo_ren_id = state.ren_id_board[oppo_color, adj_xy]
-    num_pseudo, idx_sum, idx_squared_sum = _count(state, oppo_color, size)
+    ren_id = state.ren_id_board[adj_xy]
+    num_pseudo, idx_sum, idx_squared_sum = _count(state, size)
+    ren_ix = jnp.abs(ren_id) - 1
     # fmt: off
-    is_atari = ((idx_sum[oppo_ren_id] ** 2) == idx_squared_sum[oppo_ren_id] * num_pseudo[oppo_ren_id])
-    single_liberty = (idx_squared_sum[oppo_ren_id] // idx_sum[oppo_ren_id]) - 1
-    is_killed = (adj_xy != -1) & (oppo_ren_id != -1) & is_atari & (single_liberty == xy)
+    is_atari = ((idx_sum[ren_ix] ** 2) == idx_squared_sum[ren_ix] * num_pseudo[ren_ix])
+    single_liberty = (idx_squared_sum[ren_ix] // idx_sum[ren_ix]) - 1
+    is_killed = (adj_xy != -1) & (ren_id * oppo_color > 0) & is_atari & (single_liberty == xy)
     state = jax.lax.fori_loop(
         0, 4,
         lambda i, s: jax.lax.cond(
             is_killed[i],
-            lambda: _remove_stones(s, oppo_ren_id[i], adj_xy[i]),
+            lambda: _remove_stones(s, ren_id[i], adj_xy[i], kou_may_occur),
             lambda: s,
         ),
         state,
     )
     # fmt: on
-
-    # 石を置く
     state = _set_stone(state, xy)
 
     # 周囲をマージ
@@ -241,7 +236,7 @@ def _not_pass_move(_state: State, _action: int, size) -> State:
 
     # コウの確認
     state = jax.lax.cond(
-        kou_occurred & state.agehama[my_color] - agehama_before == 1,
+        kou_may_occur & state.agehama[my_color_ix] - agehama_before == 1,
         lambda: state,
         lambda: state.replace(kou=jnp.int32(-1)),  # type:ignore
     )
@@ -249,33 +244,11 @@ def _not_pass_move(_state: State, _action: int, size) -> State:
     return state.replace(reward=jnp.zeros(2, dtype=jnp.float32))  # type: ignore
 
 
-def _remove_around_xy(i, state: State, xy, size):
-    adj_xy = _neighbour(xy, size)[i]
-    is_off = adj_xy == -1
-    is_opp_ren = state.ren_id_board[_opponent_color(state), adj_xy] != -1
-    #
-    oppo_color = _opponent_color(state)
-    oppo_ren_id = state.ren_id_board[oppo_color, adj_xy]
-
-    num_pseudo, idx_sum, idx_squared_sum = _count(state, oppo_color, size)
-
-    # fmt: off
-    is_atari = ((idx_sum[oppo_ren_id] ** 2) == idx_squared_sum[oppo_ren_id] * num_pseudo[oppo_ren_id])
-    single_liberty = (idx_squared_sum[oppo_ren_id] // idx_sum[oppo_ren_id]) - 1
-    # fmt: on
-
-    return jax.lax.cond(
-        (~is_off) & is_opp_ren & is_atari & (single_liberty == xy),
-        lambda: _remove_stones(state, oppo_ren_id, adj_xy),
-        lambda: state,
-    )
-
-
 def _merge_around_xy(i, state: State, xy, size):
     my_color = _my_color(state)
     adj_xy = _neighbour(xy, size)[i]
     is_off = adj_xy == -1
-    is_my_ren = state.ren_id_board[my_color, adj_xy] != -1
+    is_my_ren = state.ren_id_board[adj_xy] * my_color > 0
     state = jax.lax.cond(
         ((~is_off) & is_my_ren),
         lambda: _merge_ren(state, xy, adj_xy),
@@ -284,78 +257,63 @@ def _merge_around_xy(i, state: State, xy, size):
     return state
 
 
-def _illegal_move(
-    _state: State,
-) -> State:
-    r: jnp.ndarray = jnp.ones(2, dtype=jnp.float32)  # type:ignore
-    return _state.replace(terminated=TRUE, reward=r.at[_state.turn % 2].set(-1))  # type: ignore
-
-
 def _set_stone(_state: State, _xy: int) -> State:
     my_color = _my_color(_state)
     return _state.replace(  # type:ignore
-        ren_id_board=_state.ren_id_board.at[my_color, _xy].set(_xy),
+        ren_id_board=_state.ren_id_board.at[_xy].set((_xy + 1) * my_color),
     )
 
 
 def _merge_ren(_state: State, _xy: int, _adj_xy: int):
     my_color = _my_color(_state)
-    my_ren_id_board = _state.ren_id_board[my_color]
-
-    new_id = my_ren_id_board[_xy]
-    adj_ren_id = my_ren_id_board[_adj_xy]
-    small_id, large_id = jnp.minimum(new_id, adj_ren_id), jnp.maximum(
-        new_id, adj_ren_id
-    )
+    new_id = jnp.abs(_state.ren_id_board[_xy])
+    adj_ren_id = jnp.abs(_state.ren_id_board[_adj_xy])
+    # fmt: off
+    small_id = jnp.minimum(new_id, adj_ren_id) * my_color
+    large_id = jnp.maximum(new_id, adj_ren_id) * my_color
+    # fmt: on
 
     # 大きいidの連を消し、小さいidの連と繋げる
     ren_id_board = jnp.where(
-        my_ren_id_board == large_id, small_id, my_ren_id_board
+        _state.ren_id_board == large_id, small_id, _state.ren_id_board
     )
 
     return _state.replace(  # type:ignore
-        ren_id_board=_state.ren_id_board.at[my_color].set(ren_id_board),
+        ren_id_board=ren_id_board,
     )
 
 
-def _remove_stones(_state: State, _rm_ren_id, _rm_stone_xy) -> State:
-    oppo_color = _opponent_color(_state)
-    surrounded_stones = _state.ren_id_board[oppo_color] == _rm_ren_id
+def _remove_stones(
+    _state: State, _rm_ren_id, _rm_stone_xy, kou_may_occur
+) -> State:
+    surrounded_stones = _state.ren_id_board == _rm_ren_id
     agehama = jnp.count_nonzero(surrounded_stones)
-    oppo_ren_id_board = jnp.where(
-        surrounded_stones, -1, _state.ren_id_board[oppo_color]
+    ren_id_board = jnp.where(surrounded_stones, 0, _state.ren_id_board)
+    kou = jax.lax.cond(
+        kou_may_occur & (agehama == 1),
+        lambda: jnp.int32(_rm_stone_xy),
+        lambda: _state.kou,
     )
-
     return _state.replace(  # type:ignore
-        ren_id_board=_state.ren_id_board.at[oppo_color].set(oppo_ren_id_board),
-        agehama=_state.agehama.at[_my_color(_state)].add(agehama),
-        kou=jnp.int32(_rm_stone_xy),  # type:ignore
+        ren_id_board=ren_id_board,
+        agehama=_state.agehama.at[_my_color_ix(_state)].add(agehama),
+        kou=kou,
     )
 
 
 def legal_actions(state: State, size: int) -> jnp.ndarray:
-    is_empty = (state.ren_id_board[BLACK] == -1) & (
-        state.ren_id_board[WHITE] == -1
-    )
+    is_empty = state.ren_id_board == 0
 
     my_color = _my_color(state)
     opp_color = _opponent_color(state)
-    my_ren = state.ren_id_board[my_color]
-    opp_ren = state.ren_id_board[opp_color]
-    my_num_pseudo, my_idx_sum, my_idx_squared_sum = _count(
-        state, my_color, size
-    )
-    opp_num_pseudo, opp_idx_sum, opp_idx_squared_sum = _count(
-        state, opp_color, size
-    )
+    num_pseudo, idx_sum, idx_squared_sum = _count(state, size)
 
+    ren_ix = jnp.abs(state.ren_id_board) - 1
     # fmt: off
-    my_in_atari = (my_idx_sum[my_ren] ** 2) == my_idx_squared_sum[my_ren] * my_num_pseudo[my_ren]
-    opp_in_atari = (opp_idx_sum[opp_ren] ** 2) == opp_idx_squared_sum[opp_ren] * opp_num_pseudo[opp_ren]
+    in_atari = (idx_sum[ren_ix] ** 2) == idx_squared_sum[ren_ix] * num_pseudo[ren_ix]
     # fmt: on
-    is_empty = (state.ren_id_board[0] == -1) & (state.ren_id_board[1] == -1)
-    has_liberty = (my_ren >= 0) & ~my_in_atari
-    kills_opp = (opp_ren >= 0) & opp_in_atari
+    has_liberty = (state.ren_id_board * my_color > 0) & ~in_atari
+    kills_opp = (state.ren_id_board * opp_color > 0) & in_atari
 
     @jax.vmap
     def is_neighbor_ok(xy):
@@ -380,11 +338,10 @@ def legal_actions(state: State, size: int) -> jnp.ndarray:
     )
 
 
-def _count(state: State, color, size):
+def _count(state: State, size):
     ZERO = jnp.int32(0)
-    is_empty = (state.ren_id_board[BLACK] == -1) & (
-        state.ren_id_board[WHITE] == -1
-    )
+    ren_id_board = jnp.abs(state.ren_id_board)
+    is_empty = ren_id_board == 0
     idx_sum = jnp.where(is_empty, jnp.arange(1, size**2 + 1), ZERO)
     idx_squared_sum = jnp.where(
         is_empty, jnp.arange(1, size**2 + 1) ** 2, ZERO
@@ -401,41 +358,36 @@ def _count(state: State, color, size):
         # fmt: on
 
     idx = jnp.arange(size**2)
-    my_ren = state.ren_id_board[color]
-
     num_pseudo, idx_sum, idx_squared_sum = _count_neighbor(idx)
-    # num_pseudo = jnp.where(my_ren >= 0, num_pseudo, ZERO)
-    # idx_sum = jnp.where(my_ren >= 0, idx_sum, ZERO)
-    # idx_squared_sum = jnp.where(my_ren >= 0, idx_squared_sum, ZERO)
 
     @jax.vmap
     def _num_pseudo(x):
-        return jnp.where(my_ren == x, num_pseudo, ZERO).sum()
+        return jnp.where(ren_id_board == (x + 1), num_pseudo, ZERO).sum()
 
     @jax.vmap
     def _idx_sum(x):
-        return jnp.where(my_ren == x, idx_sum, ZERO).sum()
+        return jnp.where(ren_id_board == (x + 1), idx_sum, ZERO).sum()
 
     @jax.vmap
     def _idx_squared_sum(x):
-        return jnp.where(my_ren == x, idx_squared_sum, ZERO).sum()
+        return jnp.where(ren_id_board == (x + 1), idx_squared_sum, ZERO).sum()
 
     return _num_pseudo(idx), _idx_sum(idx), _idx_squared_sum(idx)
 
 
 def get_board(state: State) -> jnp.ndarray:
-    board = jnp.full_like(state.ren_id_board[BLACK], 2)
-    board = jnp.where(state.ren_id_board[BLACK] != -1, 0, board)
-    board = jnp.where(state.ren_id_board[WHITE] != -1, 1, board)
+    board = jnp.ones_like(state.ren_id_board, dtype=jnp.int8) * 2
+    board = jnp.where(state.ren_id_board > 0, 0, board)
+    board = jnp.where(state.ren_id_board < 0, 1, board)
     return board  # type:ignore
 
 
 def show(state: State) -> None:
     print("===========")
     for xy in range(state.size * state.size):
-        if state.ren_id_board[BLACK][xy] != -1:
+        if state.ren_id_board[xy] > 0:
             print(" " + BLACK_CHAR, end="")
-        elif state.ren_id_board[WHITE][xy] != -1:
+        elif state.ren_id_board[xy] < 0:
             print(" " + WHITE_CHAR, end="")
         else:
             print(" " + POINT_CHAR, end="")
@@ -446,39 +398,55 @@ def show(state: State) -> None:
 
 def _show_details(state: State) -> None:
     show(state)
-    print(state.ren_id_board[BLACK].reshape((5, 5)))
-    print(state.ren_id_board[WHITE].reshape((5, 5)))
+    print(state.ren_id_board.reshape((5, 5)))
     print(state.kou)
 
 
 def _my_color(_state: State):
-    return jnp.int32(_state.turn % 2)
+    return jnp.int32([1, -1])[_state.turn % 2]
+
+
+def _my_color_ix(_state: State):
+    return _state.turn % 2
 
 
 def _opponent_color(_state: State):
-    return jnp.int32((_state.turn + 1) % 2)
+    return jnp.int32([-1, 1])[_state.turn % 2]
 
 
-def _kou_occurred(_state: State, xy: int) -> jnp.ndarray:
+def _opponent_color_ix(_state: State):
+    return (_state.turn + 1) % 2
+
+
+def _kou_may_occur(_state: State, xy: int) -> jnp.ndarray:
     size = _state.size
     x = xy // size
     y = xy % size
     oob = jnp.bool_([x - 1 < 0, x + 1 >= size, y - 1 < 0, y + 1 >= size])
     oppo_color = _opponent_color(_state)
-    is_occupied = _state.ren_id_board[oppo_color][_neighbour(xy, size)] != -1
-    return (oob | is_occupied).all()
+    is_occupied_by_opp = (
+        _state.ren_id_board[_neighbour(xy, size)] * oppo_color > 0
+    )
+    return (oob | is_occupied_by_opp).all()
+
+
+def _count_point(state, size):
+    # NEED FIX: Japanese rule → Tromp-Taylor rule
+    return jnp.array(
+        [
+            _count_ji(state, 1, size)
+            + jnp.count_nonzero(state.ren_id_board > 0),
+            _count_ji(state, -1, size)
+            + jnp.count_nonzero(state.ren_id_board < 0),
+        ],
+        dtype=jnp.float32,
+    )
 
 
 def _get_reward(_state: State, _size: int) -> jnp.ndarray:
-    def count_ji(color):
-        return (
-            _count_ji(_state, color, _size) - _state.agehama[(color + 1) % 2]
-        )
-
-    count_ji = jax.vmap(count_ji)
-    score = count_ji(jnp.array([BLACK, WHITE]))
+    score = _count_point(_state, _size)
     r = jax.lax.cond(
-        score[BLACK] - _state.komi > score[WHITE],
+        score[0] - _state.komi > score[1],
         lambda: jnp.array([1, -1], dtype=jnp.float32),
         lambda: jnp.array([-1, 1], dtype=jnp.float32),
     )
@@ -498,9 +466,9 @@ def _neighbours(size):
 
 
 def _count_ji(state: State, color: int, size: int):
-    board = jnp.zeros_like(state.ren_id_board[0])
-    board = jnp.where(state.ren_id_board[color] >= 0, 1, board)
-    board = jnp.where(state.ren_id_board[1 - color] >= 0, -1, board)
+    board = jnp.zeros_like(state.ren_id_board)
+    board = jnp.where(state.ren_id_board * color > 0, 1, board)
+    board = jnp.where(state.ren_id_board * color < 0, -1, board)
     # 0 = empty, 1 = mine, -1 = opponent's
 
     neighbours = _neighbours(size)
